import QtQuick 2.0
import QtQuick.Controls 2.0
import QtQuick.Controls 1.4 as OQQC
import QtQuick.Controls.Styles 1.4
import com.carmenta 1.0

Item {
    id: window
    width: 800
    height: 480
    property string sidebarColor: "#303030"
    property string alternateColor: "#e0e0e0"  
    property int sidebarWidth: 50
    MapControl {
        objectName: "mapControl"
        width: parent.width - sidebarWidth
        height: parent.height
    }

    Rectangle {
        id: layers
        width: 200
        x: parent.width // - width
        y: 0
        height: parent.height
        color: alternateColor
        

        OQQC.TreeView {
            model: layerModel
            height: parent.height
            backgroundVisible: false
            headerVisible: false

            OQQC.TableViewColumn {
                title: "Layers"
                role: "display"
                resizable: false
            }

            itemDelegate: Row {
                Button{
                    width: 20
                        checkable: true
                        checked: layerModel.isChecked(styleData.index)
                        display: AbstractButton.IconOnly
                        icon.source: checked ? "eye.svg" : "eye-off.svg"
                        icon.color: sidebarColor
                        background: Rectangle {
                            color: alternateColor
                        }
                        anchors.verticalCenter: parent.verticalCenter
                        padding: 0
                        MouseArea {
                             width: parent.parent.parent.width
                             height: parent.height
                             onClicked: parent.toggle()
                        }
                        onCheckedChanged: {
                                layerModel.onCheckedChanged(styleData.index, checked); 
                        }
                    }
                    Text {
                        text: styleData.value
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

            rowDelegate: Rectangle {
                color: alternateColor
                height: 25
            }
        }

        states: [
            State {
                when: (window.state === "LAYERS")
                PropertyChanges {
                    target: layers
                    x: parent.width - width - sidebarWidth
                }
            },
            State {
                when: (window.state !== "LAYERS")
                PropertyChanges {
                    target: layers
                    x: parent.width - sidebarWidth
                }
            }
        ]

        transitions: Transition {
            NumberAnimation {
                properties: "x"
                easing.type: Easing.InOutQuad
            }
        }
    }

    Rectangle {
        id: draw
        width: 200
        x: parent.width
        y: 0
        height: parent.height
        color: alternateColor



            ListModel {
                id: drawButtonsModel

                ListElement {
                    name: "Draw Point"
                    icon_name: "map-marker.svg"
                    draw_type: "point"
                }

                ListElement {
                    name: "Draw Line"
                    icon_name: "vector-line.svg"
                    draw_type: "line"
                }

                ListElement {
                    name: "Draw Polygon"
                    icon_name: "vector-polygon.svg"
                    draw_type: "polygon"
                }
            }



                ListView {
                    model: drawButtonsModel //setting previously defined ListModel as model
                    height: 50*drawButtonsModel.count
                    width: parent.width

                    id: drawButtonsListView

                    delegate: Row {
                        height:  50
                       spacing: 5
                       Button {
                           width: 50
                           checkable: true
                           checked: true
                           anchors.verticalCenter: parent.verticalCenter
                           display: AbstractButton.IconOnly
                           icon.source: icon_name //icon source
                           icon.color: sidebarColor
                           background: Rectangle {
                               color: alternateColor
                           }
                           MouseArea {
                                id: mouseAreaDraw
                                width: parent.parent.parent.width
                                height: parent.height
                                onClicked: {
                                    controller.drawOnMap(drawButtonsModel.get(index).draw_type)
                                            }
                                Rectangle {
                                    anchors.fill: mouseAreaDraw
                                    color: mouseAreaDraw.pressed? sidebarColor : alternateColor
                                    opacity: 0.3
                                }
                            }
                       }
                       Text {
                           text: name //display name
                           anchors.verticalCenter: parent.verticalCenter
                       }
                   }
               }


               ListView {
                   model: myTacticalModel // tactical model that we will instantiate in C++
                   anchors.top: drawButtonsListView.bottom
                   height: parent.height - drawButtonsListView.height

                   delegate: Row {
                       height: 50
                      spacing: 5
                      Button {
                          width: 60
                          height: 60
                          checkable: true
                          checked: true
                          display: AbstractButton.IconOnly
                          icon.source: "image://tactical/" + index // our model will act as a provider and provide the images
                          icon.cache: false
                          icon.color: sidebarColor
                          background: Rectangle {
                              color: alternateColor
                          }
                          anchors.verticalCenter: parent.verticalCenter

                          MouseArea {
                               id: mouseAreaTacticalDraw
                               width: 200
                               height: parent.height
                               onClicked: {
                                  controller.tacticalDrawOnMap(myTacticalModel.getSidcFromIndex(index)) // callback consists in getting the SIDC from the model and forwarding it to the controller
                                           }
                               Rectangle {
                                   anchors.fill: mouseAreaTacticalDraw
                                   color: mouseAreaTacticalDraw.pressed? sidebarColor : alternateColor
                                   opacity: 0.3
                               }
                           }
                      }
                      Text {
                          text: model.display // using the display role of the model to display
                          anchors.verticalCenter: parent.verticalCenter
                      }
                  }
               }




        states: [
            State {
                when: (window.state === "DRAW")
                PropertyChanges {
                    target: draw
                    x: parent.width - width - sidebarWidth
                }
            },
            State {
                when: (window.state !== "DRAW")
                PropertyChanges {
                    target: draw
                    x: parent.width - sidebarWidth
                }
            }
        ]

        transitions: Transition {
            NumberAnimation {
                properties: "x"
                easing.type: Easing.InOutQuad
            }
        }
    }

    function togglePanel(panel, checked) {
        if (checked) {
            window.state = panel
        } else {
            window.state = "HIDDEN"
        }
    }

    Rectangle {
        id: sidePanel
        x: parent.width - sidebarWidth
        y: 0
        width: sidebarWidth
        height: parent.height
        color: sidebarColor

        ButtonGroup {
            id: buttonGroup
        }

        ListModel {
            id: buttonModel

            ListElement {
                name: "Layers"
                icon_source: "layers.svg"
                click: "LAYERS"
            }
            ListElement {
                name: "Draw"
                icon_source: "pencil.svg"
                click: "DRAW"
            }
        }
        ListView {
            anchors.fill: parent
            model: buttonModel

            delegate: Button {

                width: 50
                checkable: true
                checked: (window.state === click)
                display: AbstractButton.IconOnly
                icon.source: icon_source
                icon.color: (checked ? sidebarColor : alternateColor)
                onToggled: togglePanel(click, checked)
                background: Rectangle {
                    color: (parent.checked ? alternateColor : sidebarColor)
                }
            }
        }
    }
    states: [
        State {
            name: "HIDDEN"
        },
        State {
            name: "LAYERS"
        },
        State {
            name: "DRAW"
        }
    ]
}

/*##^##
Designer {
    D{i:0;formeditorZoom:0.33}
}
##^##*/

