#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QObject>
#include <Carmenta/Engine/Controls/QtQuickMapControl.hpp>

class Controller : public QObject
{
	Q_OBJECT
public:
	// Get singleton instance.
	static Controller& instance();

	// Initialize the Controller instance with the root of the QML scene.
	void initialize(QObject* root);

	// Zoom in.
	Q_INVOKABLE void zoomIn();

	// Zoom out.
	Q_INVOKABLE void zoomOut();

	// Load a map configuration and display the first view.
	Q_INVOKABLE void loadConfiguration(const QString& fileName);

    // Called when pressing on a button for simple drawings (point, line or polygon)
    Q_INVOKABLE void drawOnMap(QString drawingType);

    // Called when pressing on a button meant to draw tactical symbols or graphics
    Q_INVOKABLE void tacticalDrawOnMap(QString sidc);

	Carmenta::Engine::ViewPtr view()
	{
		return mapControl->view();
	}

	void updateView()
	{
		return mapControl->updateView();
	}

private:

	// Constructor.
	explicit Controller(QObject* parent = nullptr);

	// The MapControl.
	Carmenta::Engine::QtQuickMapControl* mapControl;

    // Initialization method for tools (standard and create).
    void initializeTools();

    // Standard tool and touchtool.
    Carmenta::Engine::ToolPtr _standardTool;
    Carmenta::Engine::TouchToolPtr _standardTouchTool;

    // Create tool and touchtool.
    Carmenta::Engine::ToolPtr _createTool;
    Carmenta::Engine::TouchToolPtr _createTouchTool;

    // Memory Dataset reference, store drawn features.
    Carmenta::Engine::MemoryDataSetPtr _drawnObjectsMemDs;

    Carmenta::Engine::CreateToolPtr _createTacticalTool;
    Carmenta::Engine::CreateTouchToolPtr _createTacticalTouchTool;
    Carmenta::Engine::MemoryDataSetPtr _tacticalDrawingsMemDs;

    // Convenience method to set the standard tool on the map control.
    void swapToStandardTool();

    // Convenience method to set the create tool with designated create mode.
    void swapToCreateTool(Carmenta::Engine::ToolCreateMode& mode, Carmenta::Engine::CreateToolParametersPtr toolParameters);

    // Event handler for Carmenta event: featureCreated.
    inline static void CARMENTA_ENGINE_STDCALL featureCreatedHandler(
        const Carmenta::Engine::EngineObjectPtr&,
        const Carmenta::Engine::FeatureCreatedEventArgs&,
        void* data)
    {
        auto controller = static_cast<Controller*>(data);
        controller->swapToStandardTool();
    }
};

// The singleton type provider function (callback).
QObject* Controller_singletontype_provider(QQmlEngine*, QJSEngine*);

#endif
