#include "layermodel.h"
#include <QVariant>

void LayerModel::initialize()
{
	ViewPtr view = Controller::instance().view();
	addChildLayerNodes(view->layers(), this->invisibleRootItem());
}

QHash<int, QByteArray> LayerModel::roleNames() const
{
	QHash<int, QByteArray> result= QStandardItemModel::roleNames();

	result[Qt::CheckStateRole] = "checkState";
	result[LayerRole] = "layer";
	result[TreeIndexRole] = "treeIndex";

	return result;
}

void LayerModel::addChildLayerNodes(LayerCollectionPtr layerSet, QStandardItem* rootNode)
{
	// Iterating on all children of the current node. All the children are on the same level.
	for (uint32_t i = 0; i < layerSet->size(); i++)
	{
		LayerPtr layer = layerSet->get(i);
		QStandardItem* layerItem = new QStandardItem(0);

		AttributeValue attr;

		String name = layer->name().c_str();
		String displayName = layer->displayName().c_str();
		if (displayName != " " && displayName != "")
			name = displayName;
		layerItem->setText(name.c_str());

		QVariant data; 
		data.setValue(layer);
		layerItem->setData(data, LayerRole);

		layerItem->setToolTip(layer->description().c_str());

		rootNode->appendRow(layerItem);

		// Store index to make it accessible in QML
		layerItem->setData(layerItem->index(), TreeIndexRole);

		layerItem->setCheckable(true);
		layerItem->setCheckState(layer->enabled() ? Qt::Checked : Qt::Unchecked);

		// If the current child can be cast to a LayerSet (which layers with several children inherit from), the function is run again on its children. 
		LayerSetPtr cLayer = dynamic_cast<LayerSet*>(layer.get());
		if (cLayer != 0)
			addChildLayerNodes(cLayer->layers(), layerItem);
	}
}

void LayerModel::onCheckedChanged(const QModelIndex& index, const bool checked)
{
	// Get the LayerItem corresponding to the model index.
	QStandardItem* layerItem = this->itemFromIndex(index);
	if (!layerItem)
		return;

	// Keeping the model in sync with the UI
    layerItem->setCheckState(checked ? Qt::Checked : Qt::Unchecked);

	// Toggle the layer enabled state
	LayerPtr layer = layerItem->data(LayerRole).value<LayerPtr>();
	layer->enabled(checked);

	// Update the view.
	Controller::instance().updateView();
}

bool LayerModel::isChecked(QModelIndex index)
{
	// Root element's index is invalid. 
	// Default value is false but it doesn't matter as it isn't visualized.
	if (index.isValid())
		return this->itemFromIndex(index)->checkState() == Qt::Checked;
	return false;
}
