#ifndef TACTICAL_SYMBOL_ITEM_H
#define TACTICAL_SYMBOL_ITEM_H

#include <QQuickView>
#include <Carmenta\Engine\Core.hpp>
#include <Carmenta\Engine\Tactical.hpp>

using namespace Carmenta::Engine;

class TacticalSymbolItem : QObject
{
	Q_OBJECT
public:
	TacticalSymbolItem();
	TacticalSymbolItem(const TacticalSymbolItem& item);
	TacticalSymbolItem(QString sidc, QString description = "");
	~TacticalSymbolItem();

	TacticalSymbolItem& operator=(const TacticalSymbolItem&);

	QImage preview; // Holds the preview Image
	QString name; // Symbol description

	inline QString getSidc() const { return _sidc; };

private:
	QString _sidc; // SIDC attribute value
	Carmenta::Engine::Atom _sidcAtom; // SIDC attribute name in CE

	QSize _previewSize; 


	// Military standard used by the symbol
	Carmenta::Engine::SymbolStandardPtr m_standard = Carmenta::Engine::SymbolStandard::milStd2525C();
	// Symbol metadata
	Carmenta::Engine::SymbolMetadataPtr m_symbolMetadata;
	// Legend item used to generate preview
	Carmenta::Engine::LegendItemPtr m_previewLegendItem = NULL;

	bool setSidc(QString sidc, QString description = "");
	// returns the preview image
	QImage computePreview(Carmenta::Engine::String sidc);
	// Initialize the legend item to prepare for previewing
	void initializePreview();
};

#endif
