#include "tacticalsymbolsmodel.h"

#include <QFile>
#include <QRegularExpression>
#include <QBuffer>

using namespace Carmenta::Engine;

TacticalSymbolsModel::TacticalSymbolsModel(QObject* parent) :
    QAbstractListModel(parent)
{
    // Read symbols from CSV file.
    QFile file("carmenta.engine/TacticalSymbols.csv");
    if (file.open(QIODevice::ReadOnly)) {
        QString f = file.readAll();
        QStringList symbols = f.split(QRegularExpression("[\r\n]"), Qt::SkipEmptyParts);
        for (int i = 0; i < symbols.count(); i++)
        {
            QStringList csv = symbols[i].split(';', Qt::SkipEmptyParts);
            QString sidc = csv.at(0);
            QString description = csv.count() > 1 ? csv.at(1) : "";
            _symbols.append(TacticalSymbolItem(sidc, description));
        }
    }
}

int TacticalSymbolsModel::rowCount(const QModelIndex& parent) const
{
    if (!parent.isValid())
    {
        return _symbols.count();
    }

    return 0;
}

QVariant TacticalSymbolsModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid())
    {
        return QVariant();
    }

    else if (role == Qt::DisplayRole)
    {
        return _symbols.at(index.row()).name;
    }

    return QVariant();
}

QString TacticalSymbolsModel::getSidcFromIndex(int index)
{
    return _symbols.at(index).getSidc();
}

QUrl imageToUrl(const QImage& image)
{
    QByteArray byteArray;
    QBuffer buffer(&byteArray);
    buffer.open(QIODevice::WriteOnly);
    image.save(&buffer, "png");
    QString base64 = QString::fromUtf8(byteArray.toBase64());
    return QString("data:image/png;base64,") + base64;
}

QUrl TacticalSymbolsModel::requestImage(const int id)
{
    return imageToUrl(_symbols.at(id).preview);
}
